#include "KshTextInput.H"

#include "Logger.H"
extern Logger logger;

KshTextInput::KshTextInput(unsigned int wdth, unsigned int maxlen,
		StrList &path, StrPairList &env):
	TextInput(wdth,maxlen),
	_command(false),
	_change(false),
	_word(false),
	_delete(false),
	_replace(false),
	_startpos(0),
	_endpos(0),
	_iteration(0),
	_path(path),
	_env(env) {
}

KshTextInput::~KshTextInput() {
}

widstat KshTextInput::input(Char chr) {
	widstat rv;
	chr.setLike(_stile);
	char c=chr.getSymbol();
	if ((chr.isEnter()) && (TextInput::input(chr)==ws_completed)) {
		_command=false;
		_change=false;
		_word=false;
		_delete=false;
		_replace=false;	
		return (ws_completed);
	}
	else if (chr.isInterrupt()) {
		_command=false;
		_change=false;
		_word=false;
		_delete=false;
		_replace=false;	
		reset();
		return (ws_accepted);
	}
	else if (_change) {
		if (chr.isEsc()) {
			_change=false;	
			_command=true;
			_word=false;
logger.log("endpos=%d index=%d",_endpos,curPos());
			while ((_endpos+1)>curPos()) {
logger.log("erasing position %d of %d",_endpos, _line.length());
				_line.erase(_endpos--);
			}
			_endpos=0;
		}
		else if (_word) {
			if (chr.isPrintable()) {
				if (_endpos>=curPos()) {
					_line.replace(chr,curSet(curPos()+1));
				}
				else {
					_line.insert(chr,curSet(curPos()+1));
				}
			}
			else if (chr.isBksp()) {
				if (curPos() > _startpos) {
					curLeft();
					_line.erase(curPos());
					_endpos--;
				}
			}
		}
		else if (c == 'w') {
			int rval;
			chr='$';
			_word=true;
			_startpos=curPos();
			if ((rval=findWordEnd(1,curPos()))>0)
				_endpos=rval;
			else
				_endpos=curPos();
			_line.replace(chr,_endpos);
		}
	}
	else if (_delete) {
		if (c == 'w') {
			int end=findWordBegin(1,curPos());
			if (end < 0) {
				end=findWordEnd(1,curPos());
				end++;
			}
			while (end > (signed)curPos()) {
				_line.erase(--end);
			}
		}
		_delete=false;	
		_command=true;
	}
	else if (_replace) {
		set(chr,curPos());
		_replace=false;	
		_command=true;
	}
	else if (_command) {
		if (c == '\\') {
			matchInPlace(_path);
			_command=false;
		}
		else if ((c == '0') || (c == '^')) {
			if (!_iteration)
				curSet(0);
			// else, add to command iteration
		}
		else if (c == '$') {
			if (!_iteration)
				curSet(length());
			// else, add to command iteration
		}
		else if (c == 'a') {
			_command=false;
			curSet(curPos()+1);
		}
		else if (c == 'A') {
			curSet(length());
			_command=false;
		}
		else if (c == 'b') {
			int idx=findWordEnd(-1,curPos()-1);
			if (idx>=0)
				curSet(idx);
		}
		else if (c == 'c') {
			_change=true;
		}
		else if (c == 'C') {
			_line.chop(curPos());
			_command=false;
		}
		else if (c == 'd') {
			_delete=true;
		}
		else if (c == 'D') {
			_line.chop(curPos());
			curLeft();
		}
		else if (c == 'e') {
			int idx=findWordEnd(1,curPos()+1);
			if (idx>=0)
				curSet(idx);
		}
		else if ((c == 'h') || (chr.isBksp())) {
			curLeft();
		}
		else if (c == 'i') {
			_command=false;
		}
		else if (c == 'I') {
			_command=false;
			curSet(0);
		}
		else if ((c == 'j') && (TextInput::_hindex)) {
			setFromHistory(--TextInput::_hindex);
		}
		else if ((c == 'k') && (setFromHistory(TextInput::_hindex+1)==0)) {
			if (TextInput::_hindex+1 <= currentHistorySize())
				TextInput::_hindex++;
		}
		else if ((c == 'l') && ((signed)curPos() < (signed)(length()-1))) {
			curRight();
		}
		else if (c == 'r') {
			_replace=true;
		}
		else if (c == 'u') {
			Text tmp;
			tmp=_line;
			_line=_undo;
			_undo=tmp;
		}
		else if (c == 'w') {
			int idx=findWordBegin(1,curPos()+1);
			if (idx>=0)
				curSet(idx);
			else if ((idx=findWordEnd(1,curPos()+1))>=0)
				curSet(idx);
		}
		else if ((c == 'x') && (length())) {
			if (curPos() > length()) {
				curLeft();
			}	
			else {
				erase(curPos());
				if ((curPos()) && (curPos()==length()))
					curSet(curPos()-1);
			}
		}
		else if (c == 'X') {
			curLeft();
			erase(curPos());
		}
	}
	else if (chr.isEsc()) {
		_command=true;
		if (curPos())
			curSet(curPos()-1);
		TextInput::_hindex=0;
	}
	else if ((rv=TextInput::input(chr))==ws_completed) {
		_undo="";
		return (ws_completed);
	}
	else if (rv!=ws_accepted) {
		return (ws_rejected);
	}
	_undo=_line;
	return (ws_accepted);
}
