#include "Logger.H"
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <time.h>

Logger:: Logger(int level, const char *fname, const char *loc, const char *suffix) {
	int
		len;
	char
		buf[256],
		slash[2]="";
 
	init(level);
	if (fname) {
		if ((len=strlen(loc)) && (loc[len]!='/'))
			slash[0]='/';slash[1]=0;
		sprintf(buf,"%s%s%s%s",loc,slash,fname,suffix);	
		if ((_fp=fopen(buf,"a")) == NULL) {
			fprintf(stderr,"Cannot open log file: %s (redirecting to stderr)",buf);
			_fp=stderr;
		}
		else {
			_fopened=true;
		}
	}	
	setbuf(_fp,NULL);
}

Logger::Logger(int level, FILE *dest) {
	init(level);
	_fp=dest;
}

Logger::~Logger() {
	if ((_fp) && (_fopened))
		fclose(_fp);
}

void Logger::init(int level) {
	_fopened=false;
	_fp=stderr;
	_timestamp=false;
	_pid=getpid();
	_level=level;
	_nl=true;
	_bpid=true;
	strcpy(_format,"%a %H:%M:%S");
	_defaultlevel=1;
}

void Logger::log(const char *txt,...) {
	va_list
		var;
	char
		buf[1024];

	va_start(var,txt);
	vsnprintf(buf,sizeof(buf),txt,var);
	va_end(var);
	if (getDefaultLevel() > getLevel())
		return;
	realLog(buf);
}

void Logger::log(int level, const char *txt, ...) {
	va_list
		var;
	char
		buf[1024];

	va_start(var,txt);
	vsnprintf(buf,sizeof(buf),txt,var);
	va_end(var);
	if 	(level > getLevel())
		return;
	realLog(buf);
}

void Logger::realLog(char *txt,const char *file, int line) {
	FILE
		*fp;
	time_t
		ttime;
	struct tm
		*tms;
	char
		tbuf[64];

	if ((!txt) || ((fp=getFilePtr())==NULL))
		return;
	if (_nl)
		for (int i=strlen(txt); txt[i-1]=='\n'; i--) txt[i-1]=0;
	tbuf[0]=0;
	if (timestamp()) {
		ttime=time(NULL);
		tms=localtime(&ttime);
		strftime(tbuf, sizeof(tbuf), getFormat(), tms);
	}
	if (_bpid||tbuf[0])
		fprintf(fp,"[");
	if (_bpid)
		fprintf(fp,"%d",_pid);
	if (_bpid&&tbuf[0])
		fprintf(fp," - ");
	if (tbuf[0])
		fprintf(fp,"%s",tbuf);
	if (_bpid||tbuf[0])
		fprintf(fp,"] ");
	if (file)
		fprintf(fp,"%s:%d: ",file,line);
	fprintf(fp,"%s",txt);	
	if (_nl)
		fprintf(fp,"\n");
}
