#ifndef _LOGGER_H_
#define _LOGGER_H_

#include <stdio.h>
#include <string.h>

//#define BLAH(OBJ,FMT) OBJ.log(__FILE__,__LINE__,FMT)
//#define BLAH(OBJ,FMT,A) OBJ.log(__FILE__,__LINE__,FMT,A)

class Logger {
	public:
		Logger(int level=1, const char *fname=NULL, const char *loc="", const char *suffix="");
		Logger(int level, FILE *dest);
		virtual ~Logger();
		virtual void log(const char *txt, ...);
		virtual void log(int level, const char *txt, ...);
		virtual void setLevel(int level) { _level=level; }
		virtual int getLevel() { return _level; }
		virtual void setDefaultLevel(int deflvl) { _defaultlevel=deflvl; }
		virtual int getDefaultLevel() { return _defaultlevel; }
		virtual int getPid() { return _pid; }
		virtual void timestampOn(const char *format=NULL) { _timestamp=true; if (format) strncpy(_format, format, sizeof(_format)); };
		virtual void timestampOff() { _timestamp=false; };
		virtual void pidOn() { _bpid=true; }
		virtual void pidOff() { _bpid=false; }
		virtual void newlineOn() { _nl=true; }
		virtual void newlineOff() { _nl=false; }
		virtual bool timestamp() { return _timestamp; };
		virtual void close() { if (_fp) fclose(_fp); _fp=NULL; }
		virtual void flush() { if (_fp) fflush(_fp); }
	protected:
		virtual char *getFormat() { return _format; }
		virtual FILE *getFilePtr() { return _fp; }
		virtual void realLog(char *txt,const char *file=NULL, int line=0);	
	private:
		Logger(const Logger &);
		Logger &operator=(const Logger &);
		void init(int level);
		FILE *_fp;
		bool _fopened;
		int _pid;
		int _level;
		int _defaultlevel;
		bool _timestamp;
		bool _bpid;
		bool _nl;
		char _format[64];	
};

#endif // _LOGGER_H_
