#ifndef __SCREEN_H__
#define __SCREEN_H__

#include <utility>
#include <string>
#include <deque>
#include <queue>
#include "Char.H"
#include "Text.H"
#include "Grid.H"
#include "Window.H"
#include "WidgetList.H"
#include <pthread.h>

#define KEY_SEQUENCE_LENGTH		(8)
#define D_INPUT_WAIT			(50000)
#define D_INPUT_HANDLER_WAIT	(50000)
#define D_REFRESH_WAIT			(100000)
#define TIMER_HACK(SCRHACK)		(500000/(SCRHACK))

class Screen {
	public:
		Screen(CharColor fore=cc_white, CharColor back=cc_black);
		~Screen();
		int init();
		int cleanup();
		void bailout();
		int width() { return (_width); };
		int height() { return (_height); };
		int colorOn();
		void colorOff();
		bool isColor() { return (_color); }
		int registerWindow(Window *win, const string name,unsigned int x=0,
				unsigned int y=0, Char activator=0x0);
		Window &getActiveWindow();
		Window &getWindow(const string name);
		int snapshot(const string &fname, const Grid &grd);
		int adjustTiming(unsigned long rwait, unsigned long iwait,
				unsigned long ihwait);
		void getTiming(unsigned long &rwait, unsigned long &iwait,
				unsigned long &ihwait);
		void resetTiming();
	private:
		pthread_mutex_t _scrlock;
		pthread_mutex_t _inplock;
		pthread_t _inputReaderThr;
		pthread_t _inputHandlerThr;
		pthread_t _updateThr;
		Grid *_grd;
		WidgetList _list;
		queue< int > _inp;
		unsigned int _width;
		unsigned int _height;
		unsigned long _input_wait;
		unsigned long _input_handler_wait;
		unsigned long _refresh_wait;
		unsigned long _timer_hack;
		unsigned int _timer;
		short _colorChart[cc_last][cc_last];
		int _usedColors;
		short _currentColor;
		short _foreDef;
		short _backDef;
		bool _color;

		Screen(const Screen &);
		Screen &operator=(const Screen &);
		void clearScr();
		int paint();
		int paint(unsigned int y, unsigned int x, const Grid &_grd);
		short getNativeColor(CharColor color);
		int setScreenByChar(Char chr);
		Char getInput();
		Char keymap(int *inp);
		static void *inputReaderRunner(Screen *this_);
		int handleInput(Char key);
		static void *inputHandlerRunner(Screen *this_);
		int update();
		static void *updateRunner(Screen *this_);
		static void sigResizeHandler(int sig);
};

#endif // __SCREEN_H__
