#include "TextArea.H"

#include "Logger.H"
extern Logger logger;

TextArea::TextArea(unsigned int wdth, unsigned int hght,
			unsigned int buf, bool bow, bool stripnl,bool readonly,
			int indent, char indent_char, bool pad):
		_width(wdth),
		_height(hght),
		_bufsize(buf?buf:hght),
		_bufpos(-1),
		_grd(NULL),
		_changed(true),
		_bow(bow),
		_stripnl(stripnl),
		_readonly(readonly),
		_indent(indent),
		_indent_char(indent_char),
		_pad(pad) {
	_scrollbar=(_bufsize>_height);
	_grd=new Grid(_width,_height);
}

TextArea::TextArea(const TextArea &tg):
		Widget(tg),
		_width(tg._width),
		_height(tg._height),
		_bufsize(tg._bufsize),
		_bufpos(tg._bufpos),
		_stile(tg._stile),
		_changed(true),
		_scrollbar(tg._scrollbar),
		_bow(tg._bow),
		_stripnl(tg._stripnl),
		_readonly(tg._readonly),
		_indent(tg._indent),
		_indent_char(tg._indent_char),
		_pad(tg._pad),
		_orig(tg._orig),
		_area(tg._area) {
	_grd=new Grid(*tg._grd);
}

TextArea::~TextArea() {
	delete (_grd);
}

widstat TextArea::input(Char chr) {
	if (chr.isUpArrow())
		scrollUp();
	else if (chr.isDownArrow())
		scrollDown();
	else if (chr.isPgUp())
		pageUp();
	else if (chr.isPgDn())
		pageDown();
	else if (chr.isHome())
		toTop();
	else if (chr.isEnd())
		toBottom();
	else if ((!_readonly) && (chr.isPrintable()))
		operator+=(chr);
	else
		return (ws_rejected);
	return (ws_accepted);
}

const Grid &TextArea::draw() {
	unsigned int
		x,
		y,
		i=0;
	Char
		c;

	tlock();
	(*_grd).fill(_stile);
	if (lines() <= _height)
		i=0;
	else
		if (_bufpos == -1)
			i=lines() - _height;
		else
			i=_bufpos;
	y=0;
	while (i < lines()) {
		for (x=0; x < _area[i].width(); x++) {
			c=_area[i][x];
			if (c.getFore()==cc_undef)
				c.setFore(_stile.getFore());
			if (c.getBack()==cc_undef)
				c.setBack(_stile.getBack());
			(*_grd).set(x,y,c);
		}
		y++;
		i++;
	}
	if (_scrollbar) {
		c=_stile;
		c.swap();
		c.tStandout();
		c='^';
		(*_grd).set(_width-1,0,c);
		c=0x20;
		for (y=1; y < (_height-1); y++) {
			(*_grd).set((_width-1),y,c);
		}
		c='v';
		(*_grd).set((_width-1),(_height-1),c);
		if (lines() > _height) {
			c='*';
			if (_bufpos == -1)
				(*_grd).set((_width-1),(_height-2),c);
			else if (_bufpos == 0)
				(*_grd).set((_width-1),1,c);
			else {
				int
					pos=_height-4;
				if (pos <= 0)
					pos=2;
				pos=((lines())/pos);
				if (pos <= 0) 
					pos=2;
				pos=(_bufpos)/pos+2;
				if (pos > (signed)_height-2)
					pos=_height-2;
				else if (pos == 0) 
					pos=1;
				(*_grd).set((_width-1),pos,c);
			}
		}
	}
	tunlock();
	_changed=false;
	return (*_grd);
}

widstat TextArea::resize(unsigned int wdth, unsigned int hght) {
	if ((_width==wdth) && (_height==hght))
		return (ws_completed);
	tlock();
	delete (_grd);
	_height=hght;
	_width=wdth;
	_grd=new Grid(_width,_height);
	_scrollbar=(_bufsize>_height);
	fitToArea();
	tunlock();
	return (ws_accepted);
}

widstat TextArea::style(Char chr) {
	_stile.setLike(chr);
	_changed=true;
	return (ws_accepted);
}

int TextArea::scrollUp() {
	if (_bufpos==0)
		return -1;
	if (_bufpos==-1)
		_bufpos=lines() - _height;
	_bufpos--;
	_changed=true;
	return 0;
}

int TextArea::scrollDown() {
	if (_bufpos==-1) 
		return -1;
	_bufpos++;
	if (_bufpos>=(signed)(lines() - _height))
		_bufpos=-1;
	_changed=true;
	return 0;
}

int TextArea::pageUp() {
	if (_bufpos==0)
		return -1;
	if (_bufpos==-1)
		_bufpos=lines() - _height;
	_bufpos-=((_height/3)*2);
	if (_bufpos < 0)
		_bufpos=0;
	_changed=true;
	return 0;
}

int TextArea::pageDown() {
	if (_bufpos==-1) 
		return -1;
	_bufpos+=((_height/3)*2);
	if (_bufpos>=(signed)(lines() - _height))
		_bufpos=-1;
	_changed=true;
	return 0;
}

int TextArea::toTop() {
	_bufpos=0;
	_changed=true;
	return 0;
}

int TextArea::toBottom() {
	_bufpos=-1;
	_changed=true;
	return 0;
}

TextArea &TextArea::operator=(const TextArea &tg) {
	_width=tg._width;
	_height=tg._height;
	_bufsize=tg._bufsize;
	_bufpos=tg._bufpos;
	tlock();
	if (_grd)
		delete _grd;
	_grd=new Grid(*tg._grd);
	tunlock();
	_stile=tg._stile;
	_changed=true;
	_scrollbar=tg._scrollbar;
	_bow=tg._bow;
	_stripnl=tg._stripnl;
	_readonly=tg._readonly;
	_indent=tg._indent;
	_indent_char=tg._indent_char;
	_pad=tg._pad;
	_orig=tg._orig;
	_area=tg._area;
	return (*this);
}

TextArea &TextArea::operator=(const Text &t) {
	clear();
	operator+=(t);
	return (*this);
}

TextArea &TextArea::operator=(Char chr) {
	clear();
	operator+=(chr);
	return (*this);
}

TextArea &TextArea::operator+=(const TextArea &tg) {
	tlock();
	for (unsigned int i=0; i < tg._orig.size(); i++) {
		addToOrig(tg._orig[i]);
		addToArea(tg._orig[i]);
	}
	tunlock();
	_changed=true;
	return (*this);
}

TextArea &TextArea::operator+=(const Text &t) {
	tlock();
	addToOrig(t);
	addToArea(t);
	tunlock();
	return (*this);
}

TextArea &TextArea::operator+=(Char chr) {
	Text
		line;

	tlock();
	line=_orig.back();
	_orig.pop_back();
	line+=chr;	
	addToOrig(line);
	addToArea(line);
	tunlock();
	return (*this);
}

void TextArea::clear() {
	tlock();
	_area.clear();
	_orig.clear();
	tunlock();
	_bufpos=-1;
	_changed=true;
}

int TextArea::addToOrig(const Text &t) {
	_orig.push_back(t);
	while (_orig.size() > _bufsize)
		_orig.pop_front();
	return 0;
}

int TextArea::addToArea(const Text &t) {
	Text
		line,
		nextline;
	int
		w=_width,
		i=lines();

	if (_scrollbar)
		w--;
	line=t;
//
//	this code will bestfit (justify) starting from the previous line
//
//	if (i) {
//		nextline=_area.back();
//		nextline.stripRight(0x20);
//		nextline+=t;	
//		line=nextline.chop(w,true);
//		if (_pad)
//			nextline.padRight(_indent_char, _width);
//		_area.pop_back();
//		_area.push_back(nextline);
//	}
	i=0;
	while (line.length() > 0) {
		nextline=line.chop(w,true,_stripnl);
		if (_indent) {
			if (i++)
				line.padLeft(_indent_char,_indent);
			else
				w=w-_indent;
		}
//		if (_pad)
//			line.padRight(_indent_char, _width);
		_area.push_back(line);
		line=nextline;
	}
	while (lines() > _bufsize) {
		_area.pop_front();
	}
	return 0;
}

void TextArea::fitToArea() {
	_area.clear();
	for (unsigned int i=0; i < _orig.size(); i++)
		addToArea(_orig[i]);
}
