#include "TextInput.H"

#include "Logger.H"
extern Logger logger;

TextInput::TextInput(unsigned int wdth, unsigned int maxlen, bool secret):
		Widget() {
	init(wdth,maxlen,secret,NULL);
}

TextInput::TextInput(unsigned int wdth, unsigned int maxlen,StrList &matchlist):
		Widget() {
	init(wdth,maxlen,false,&matchlist);
}

void TextInput::init(unsigned int wdth, unsigned int maxlen, bool secret,
		StrList *matchlist) {
	_secret=secret;
	_stile=0x20;
	_cursor=0x20;
	_focused=false;
	_width=wdth;
	_index=0;
	_hindex=0;
	_changed=true;
	_matchlist=matchlist;
	_maxhist=20;
	if (maxlen)
		_maxlen=maxlen;
	else
		_maxlen=_width-1;
	if (_matchlist)
		_enhanced=true;
	else 
		_enhanced=false;
	_stile.setFore(cc_black),
	_stile.setBack(cc_white);
	_cursor=_stile;
	_grd=new Grid(_width,1);
	(*_grd).fill(_stile);
	_line.style(_stile);
}

TextInput::~TextInput() {
	delete _grd;
}

widstat TextInput::gotFocus() {
	if (!_cursor.isBlink()) {
		_cursor.tReverse();
		_cursor.tBlink();
	}
	_focused=true;
	_changed=true;
	return (ws_accepted);
}

void TextInput::lostFocus() {
	if (_cursor.isBlink()) {
		_cursor.tReverse();
		_cursor.tBlink();
	}
	_focused=false;
	_changed=true;
}

widstat TextInput::input(Char chr) {
	chr.setFore(_stile.getFore());
	chr.setBack(_stile.getBack());
	while (_index > _line.length()) {
		Char b=chr; b=0x20;
		_line+=b;
	}
	if (_enhanced) {
		if (chr.getSymbol() == 0x09) {
			matchInPlace(*_matchlist,-1);
			_changed=true;
			return ws_accepted;
		}
		else if ((chr.isUpArrow()) && (chr.isControl()) &&
			(setFromHistory(_hindex+1)==0)) {
			if (_hindex+1 <= currentHistorySize())
				_hindex++;
			_changed=true;
			return ws_accepted;
		}
		else if ((chr.isDownArrow()) && (chr.isControl()) && (_hindex)) {
			setFromHistory(--_hindex);
			_changed=true;
			return ws_accepted;
		}
	}
	if (chr.isEnter()) {
		_history.push_front(get());
		while (_history.size() > _maxhist) {
			_history.pop_back();
		}
		return (ws_completed);
	}
	else if (chr.isLeftArrow()) {
		curLeft();
	}
	else if (chr.isRightArrow()) {
		curRight();
	}
	else if ((chr.isBksp()) && (_index)) {
		tlock();
		_line.erase(--_index);	
		tunlock();
	}
	else if (chr.isPrintable()) {
		if (_index >= _maxlen) {
			return (ws_rejected);
		}
		tlock();
		if (_index!=_line.length())
			_line.insert(chr,_index);
		else
			_line+=chr;
		_index++;
		tunlock();
	}
	else  {
		return ws_rejected;
	}
	_changed=true;
	return ws_accepted;
}

const Grid &TextInput::draw() {
	Char
		t;
	unsigned int
		i=0,
		j=0;

	if (_index > (_width-1))
		j=_index-_width+1;
	t.setLike(_stile);
	tlock();
	while ((i < _width) && (j < _line.length())) {
		if (_secret)
			t='*';
		else
			t=_line[j];
		if (j==_index)
			t.setLike(_cursor);
		(*_grd).set(i,0,t);
		i++;
		j++;
	}
	if (j==_index) {
		t=0x20;
		t.setLike(_cursor);
		(*_grd).set(i,0,t);
		i++;
	}
	while (i < _width) {
		t=_stile;
		if (i==_index)
			t.setLike(_cursor);
		(*_grd).set(i,0,t);	
		i++;
	}
	tunlock();
	_changed=false;
	return (*_grd);
}

widstat TextInput::style(Char stile) {
	_stile.setLike(stile);
	_cursor.setLike(stile);
	_cursor.tReverse();
	_changed=true;
	return (ws_accepted);
}

void TextInput::reset() {
	_line="";
	_index=0;
	_changed=true;
}

void TextInput::set(const string &txt) {
	_line.setText(txt);
	_index=_line.length();
	_changed=true;
}

void TextInput::set(const Text &txt) {
	_line=txt;
	_index=_line.length();
	_changed=true;
}

void TextInput::set(const Char chr, int pos) {
	if (pos > (signed) _maxlen)
		return;
	if (pos == -1) {
		_line+=chr;
		_index=_line.length();
	}
	else {
		_line.replace(chr,pos);
	}
	_changed=true;
}

int TextInput::setFromHistory(unsigned int idx) {
	if (idx==0) {
		set(_tmpline);
		return 0;
	}
	else if (--idx < _history.size()) {
		set(_history[idx]);
		return 0;
	}
	return -1;
}

int TextInput::findWordBegin(int word, unsigned int position) {
	unsigned int
		end,
		cur=0;
	bool
		backward=false,
		inalph=false,
		inpunc=false;

	if (word < 0) {
		end=0;
		word=abs(word);
		backward=true;
	}
	else if (word > 0) {
		word++;
		end=length();
		if (position>end)
			return -1;
	}
	else {
		return -2;
	}
	if ((backward) && (position==0))
		return 0;
	while (position != end) {
		if ((_line[position].isAlNum()) || (_line[position].getSymbol()=='_')) {
			if (inalph) {
				if (backward)
					position--;
				else
					position++;
				continue;
			}
			cur++;
			inalph=true;
			inpunc=false;
		}
		else if (_line[position].isBlank()) {
			if (!((inalph)||(inpunc))) {
				if (backward)
					position--;
				else
					position++;
				continue;
			}
			inalph=false;
			inpunc=false;
		}
		else {		// punctuation
			if (inpunc) {
				if (backward)
					position--;
				else
					position++;
				continue;
			}
			cur++;
			inalph=false;
			inpunc=true;
		}
		if ((signed)cur==word)
			return position;
		if (backward)
			position--;
		else
			position++;
	}
	return -3;
}

int TextInput::findWordEnd(int word, unsigned int position) {
	unsigned int
		end,
		cur=0,
		lmark=0;
	bool
		backward=false,
		inalph=false,
		inpunc=false;

logger.log("in findWordEnd()");
	if (word < 0) {
		end=0;
		word=abs(word);
		backward=true;
	}
	else if (word > 0) {
		end=length();
		if (position>end)
			return -1;
	}
	else {
		return -2;
	}
	if ((backward) && (position==0)) {
		return 0;
	}
	while (position != end) {
		if ((_line[position].isAlNum()) || (_line[position].getSymbol()=='_')) {
			if (inalph) {
				if (backward)
					position--;
				else
					position++;
				continue;
			}
			else if (inpunc) {
				if (backward)
					lmark=position+1;
				else
					lmark=position-1;
			}
			cur++;
			inalph=true;
			inpunc=false;
		}
		else if (_line[position].isBlank()) {
			if (!((inalph)||(inpunc))) {
				if (backward)
					position--;
				else
					position++;
				continue;
			}
			else {
				if (backward)
					lmark=position+1;
				else
					lmark=position-1;
			}
			inalph=false;
			inpunc=false;
		}
		else {		// punctuation
			if (inpunc) {
				if (backward)
					position--;
				else
					position++;
				continue;
			}
			else if (inalph) {
				if (backward)
					lmark=position+1;
				else
					lmark=position-1;
			}
			cur++;
			inalph=false;
			inpunc=true;
		}
		if ((signed)cur==(word+1))
			return (lmark);
		if (backward)
			position--;
		else
			position++;
	}
	if ((signed)cur==word) {
		if (backward)
			return (position?position+1:0);
		else
			return (position-1);
	}
	return -3;
}

int TextInput::match(const string &str, const StrList &matchlist, string &comp){
	unsigned int
		signif;
	int
		first=-1,
		freq=0;
logger.log("attempting to match on %s",str.c_str());
	for (unsigned int i=0; i < matchlist.size(); i++) {
		if (!strncasecmp(matchlist[i].c_str(),str.c_str(),str.length())) {
			if (first==-1)
				first=i;
			freq++;
		}
		else if (freq)
			break;
	}
	if (!freq) {
		return -1;
	}
	if (freq==1) {
		comp=matchlist[first];
		return 0;
	}
	comp=str;
	signif=comp.length();
	while (signif < matchlist[first].length()) {
		for (int x=first; x < first+freq-1; x++) {
			if (matchlist[x][signif] != matchlist[x+1][signif])
				return 1;
		}
		comp+=matchlist[first][signif++];
	}
	return 1;
}

int TextInput::matchInPlace(const StrList &matchlist,int idxoffset) {
	Char chr=_stile;
	string tmp;      
	int strt=
		findWordEnd(-1,(((signed)_index+idxoffset) < 0)?0:(_index+idxoffset));
	int end=
		findWordEnd(1,(((signed)_index+idxoffset) < 0)?0:(_index+idxoffset));
	if ((strt >= 0) && (end >= 0)) {
		int i;
		string ac;
		for (i=strt; i<=end; i++) {
			tmp+=_line[i].getSymbol();
		}
		i=match(tmp,matchlist,ac);
		if (i >= 0) {         
			for (unsigned int x=(end+1); x < (strt+ac.length()); x++) {
				chr=ac[x-strt];
				_line.insert(chr,x);
				curRight();
			}
			
			if (!idxoffset)
				curRight();
//			_line.setText(ac);
			if (i == 0)
				curRight();
		}
	}
	return 0;
}
