#ifndef __WIDGETLIST_H__
#define __WIDGETLIST_H__

#include <string>
#include <deque>
#include <utility>
#include "Widget.H"

using namespace std;

typedef struct widdata_ {
	Widget *widp;
	string name;
	int x;
	int y;
	bool moved;
	bool visible;
	Char activator;
} widdata;

class WidgetList {
	public:
		WidgetList() { pthread_mutex_init(&_lock,NULL); }
		~WidgetList() { pthread_mutex_destroy(&_lock); }
		int add(const string title, Widget *wgt, int x=0, int y=0,
				Char activator=0x0);
		int remove(const string &title);
		Widget *get(unsigned int idx=0);
		Widget *get(unsigned int idx, string &title);
		Widget *get(const string title);
		Widget *get(unsigned int idx, string &title, int &x, int &y,
				bool &mvd, bool &visible, Char &activator);
		Widget *get(const string &title, unsigned int &idx, int &x, int &y,
				bool &mvd, bool &visible, Char &activator);
		int getIndex(const string &title);
		int show(const string title);
		int show(unsigned int idx);
		int hide(const string title);
		int hide(unsigned int idx);
		bool isVisible(const string title);
		bool isVisible(unsigned int idx);
		int moveBy(const string title, int x, int y);
		int moveBy(unsigned int idx, int x, int y);
		int moveTo(const string title, int x, int y);
		int moveTo(unsigned int idx, int x, int y);
		int moved(const string title);
		int moved(unsigned int idx);
		int toFront(const string title);
		int toFront(unsigned int idx);
		int toBack(const string title);
		int toBack(unsigned int idx);
		int size() { return (_list.size()); }
	private:
		WidgetList(const WidgetList &);
		WidgetList &operator=(const WidgetList &);
		deque<widdata> _list;
		int tlock() const { pthread_mutex_lock(&_lock); return 0; }
		int tunlock() const { pthread_mutex_unlock(&_lock); return 0; }
		mutable pthread_mutex_t _lock;

};

#endif // __WIDGETLIST_H__
