#include "Window.H"
#include "TextArea.H"

#include "Logger.H"
extern Logger logger;


//int Window::brdrsym_top=	'-';
int Window::brdrsym_top=	0xAF;
int Window::brdrsym_side=	'|';
int Window::brdrsym_bot=	'_';
int Window::brdrsym_ulc=	'|';
int Window::brdrsym_urc=	'|';
int Window::brdrsym_llc=	'|';
int Window::brdrsym_lrc=	'|';
int Window::brdrsym_tfl=	'|';
int Window::brdrsym_tfr=	'|';
int Window::brdrsym_shadow=	' ';


Window::Window(int wdth, int hght, bool brdr, bool shdw, const string ttl):
		_width(wdth),
		_height(hght),
		_border(brdr),
		_shadow(shdw),
		_rebld(true),
		_stile(0x20),
		_wtitle(ttl) {
	pthread_mutex_init(&_gridlock,NULL);
	if (_shadow)
		_grd=new Grid(_width+1, _height+1);
	else
		_grd=new Grid(_width, _height);
}

Window::~Window() {
	delete (_grd);
	pthread_mutex_destroy(&_gridlock);
}

widstat Window::gotFocus() {
	Widget
		*wp;
	string
		name;

	for (int i=0; i < _widlist.size(); i++) {
		wp=_widlist.get(i,name);	
		if ((wp->inFocus())==ws_accepted) {
			_focwid=name;
			return (ws_accepted);
		}
	}
	return (ws_rejected);
}

void Window::lostFocus() {
	Widget
		*wp=_widlist.get(_focwid);	

	if (wp) {
		wp->outFocus();
		_focwid="";
	}
}

widstat Window::input(Char inp) {
	Widget
		*wp;
	for (int i=0; i < _widlist.size(); i++) {
		wp=_widlist.get(i);	
		if (wp->input(inp) == ws_accepted) {
			return (ws_accepted);
		}
	}
	return ws_invalid;
}

bool Window::update() {
	Widget
		*wp=NULL;
	Char
		activator;
	int
		i,
		x,
		y;
	string
		name;
	bool
		winmv=false,
		vis=true,
		cng=false;
        
	if (!_rebld) {
		for (i=_widlist.size(); i; i--) {
			wp=_widlist.get(i-1,name,x,y,winmv,vis,activator);
			if (winmv) {
				_rebld=true;
				break;
			}
		}
	}
	if (_rebld) {
		clearWin();
	}
	pthread_mutex_lock(&_gridlock);
	for (i=_widlist.size(); i; i--) {
		wp=_widlist.get(i-1,name,x,y,winmv,vis,activator);
		if (!vis)
			continue;
		if ((_rebld) || (cng)) {
			(*_grd).set(x,y,(*wp).draw());
			if (winmv)
				_widlist.moved(i-1);
		}
		else if ((*wp).update()) {
			(*_grd).set(x,y,(*wp).draw());
			cng=true;
		}
	}
	cng=(cng || _rebld);

//		Char chr=_stile;
//		chr.setFore(cc_black);
//		chr.tBold();
//		chr='X';
//		(*_grd).set(width()-1,0,chr);

	if (cng) {
		if (_border)
			drawBorder();
		if (_shadow)
			drawShadow();
	}
	pthread_mutex_unlock(&_gridlock);
	_rebld=false;
	return (cng);
}

widstat Window::style(Char stile) {
	_stile.setLike(stile);	
	_rebld=true;
	return (ws_accepted);
}

int Window::add(string name, Widget *wgt, unsigned int xoff,unsigned int yoff,
		Char activator) {
	return (_widlist.add(name,wgt,xoff,yoff,activator));
}

int Window::remove(const string &name) {
	return (_widlist.remove(name));
}

const Widget *Window::get(unsigned int idx) {
	return (_widlist.get(idx));
}

const Widget *Window::get(const string &name) {
	return (_widlist.get(name));
}

void Window::open() {
}

void Window::close() {
}

void Window::clearWin() {
	pthread_mutex_lock(&_gridlock);
	(*_grd).fill(_stile);
	pthread_mutex_unlock(&_gridlock);
}

void Window::drawBorder() {
	int
		i;
	Char
		c=_stile;

	c.tBold();
	c=brdrsym_top;
	for (i=1; i < (_width-1); i++)
		(*_grd).set(i,0,c);
	if (_wtitle.length()) {
		c=brdrsym_tfl;
		(*_grd).set((_width-_wtitle.length())/2-1,0,c);
		for (i=0; i < (signed)_wtitle.length(); i++) {
			c=_wtitle[i];
			(*_grd).set((_width-_wtitle.length())/2+i,0,c);
		}
		c=brdrsym_tfr;
		(*_grd).set((_width-_wtitle.length())/2+i,0,c);
	}
	c=brdrsym_side;
	for (i=1; i < (_height-1); i++) {
		(*_grd).set(0,i,c);
		(*_grd).set((_width-1),i,c);
	}
	c=brdrsym_bot;
	for (i=1; i < (_width-1); i++)
		(*_grd).set(i,_height-1,c);
	c=brdrsym_ulc;
	(*_grd).set(0,0,c);
	c=brdrsym_urc;
	(*_grd).set(_width-1,0,c);
	c=brdrsym_llc;
	(*_grd).set(0,_height-1,c);
	c=brdrsym_lrc;
	(*_grd).set(_width-1,_height-1,c);
}

void Window::drawShadow() {
	int
		i;
	Char
		c;

	c.setFore(cc_black);
	c.setBack(cc_black);
	c.tStandout();
	c=brdrsym_shadow;
	for (i=1; i < (_height); i++)
		(*_grd).set(_width,i,c);
	for (i=1; i < (_width+1); i++)
		(*_grd).set(i,_height,c);
	c.setTransparent();
	(*_grd).set(_width,0,c);
	(*_grd).set(0,_height,c);
}
