#include "YahooMessage.H"
#include "StrUtil.H"

#include "Logger.H"

extern Logger logger;

bool isRoomPacket(const char *);

char YahooMessage::_wordlist[][32] = {
	"font","/font",
	"fade","/fade",
	"alt","/alt",
	"b","/b",
	"u","/u",
	"i","/i",
	"red","/red",
	"orange","/orange",
	"yellow","/yellow",
	"green","/green",
	"blue","/blue",
	"purple","/purple",
	"black","/black",
	"white","/white",
	""
};

char YahooMessage::_r_delimitor[8] = {
	0x02, 0x30, 0x02, 0x20, 0x02, 0x30, 0x02, 0x0
	//BEM 0x02, 0x30, 0x02, 0x20, 0x02, 0x30, 0x02, 0x30, 0x02, 0x20, 0x0
};

// Added this to catch Webcams. Note the 0x31.
// Fixes the userlist garbage. BEM
char YahooMessage::_r_delimitor2[8] = {
	0x02, 0x31, 0x02, 0x20, 0x02, 0x30, 0x02, 0x0
	//BEM 0x02, 0x31, 0x02, 0x20, 0x02, 0x30, 0x02, 0x30, 0x02, 0x20, 0x0
};

// Yahoo Protocol change causes the above delimitors
// to not catch everything. Just a hack to get by.
// Fixes the userlist garbage. BEM
char YahooMessage::_r_delimitor3[6] = {
	0x02, 0x30, 0x02, 0x20, 0x02, 0x0
};

// Yahoo Protocol change causes the above delimitors
// to not catch everything. Just a hack to get by.
// Fixes the userlist garbage. BEM
char YahooMessage::_r_delimitor4[6] = {
	0x02, 0x31, 0x02, 0x20, 0x02, 0x0
};

// This is getting crazy. Need to use regex. BEM
// More Yahoo protocol changes.
char YahooMessage::_r_delimitor5[4] = {
	0x02, 0x20, 0x02, 0x0
};

YahooMessage::YahooMessage(const YahooPacket &pkt, time_t tme):
		_ptype(pkt_undefined),
		_pflag(YH_DFLAG),
		_mtype(YH_MSG_NULL_F),
		_time(tme) {
	string
		tmp;
	_ptype=pkt.getType();
	_pflag=pkt.getFlag();
	assign(pkt,tmp);
}

YahooMessage::YahooMessage(const YahooPacket &pkt, const string &user,
			time_t tme):
		_ptype(pkt_undefined),
		_pflag(YH_DFLAG),
		_mtype(YH_MSG_NULL_F),
		_time(tme) {

	_ptype=pkt.getType();
	_pflag=pkt.getFlag();
	assign(pkt,user);
}

YahooMessage::YahooMessage(const YahooMessage &ym):
		_ptype(ym._ptype),
		_pflag(ym._pflag),
		_mtype(ym._mtype),
		_who(ym._who),
		_room(ym._room),
		_desc(ym._desc),
		_msg(ym._msg),
		_hdr(ym._hdr),
		_list(ym._list) {
}

YahooMessage::~YahooMessage() {
}

YahooMessage &YahooMessage::operator=(const YahooMessage &ym) {
		_ptype=(ym._ptype);
		_pflag=(ym._pflag);
		_mtype=(ym._mtype);
		_who=(ym._who);
		_room=(ym._room);
		_desc=(ym._desc);
		_msg=(ym._msg);
		_hdr=(ym._hdr);
		_list=(ym._list);
		return (*this);
}

int YahooMessage::assign(const YahooPacket &pkt,const string &user) {
	char hdrbuf[64];
	char *hptr;

	snprintf(hdrbuf,sizeof(hdrbuf),"pkt hdr: %s %X %X %X %X\n",pkt.getIdent().c_str(),
			pkt.getVer(), pkt.getType(),pkt.getFlag(),pkt.getLen());
	_hdr=hdrbuf;
	if (_pflag) {
		_mtype|=YH_MSG_ERR_F;	
	}
	switch(_ptype) {
		case pkt_login:
			parse(pkt.getPayload(),pkt.getLen(),&_who,1,&_msg,2);
			_mtype|=YH_MSG_LOGIN_F;	
			break;
		case pkt_logout:
			//needs work
			parse(pkt.getPayload(),pkt.getLen(),&_who,1,&_msg,2);
			_mtype|=YH_MSG_LOGOUT_F;	
			break;
		case pkt_room_enter:
			parse(pkt.getPayload(),pkt.getLen(),&_room,1,&_desc,2,&_who,5);
			if (((hptr=strstr(_who.c_str(),_r_delimitor))!=NULL) &&
				(strstr(hptr+1,_r_delimitor) !=NULL)) {
				_mtype|=YH_MSG_ROOMLIST_F;
				//hack this out for now
				string
					tmp;
				char
					*ptr, *cptr, *myptr,
					buf[_who.length()+1];

				strcpy(buf,_who.c_str());
				myptr=buf;
				// Fixes userlist garbage. BEM
				while ((cptr=strstr(myptr,_r_delimitor2))!=NULL) {
					*cptr=*cptr++;
					*cptr=0x30;
				}
				ptr=buf;
				while ((cptr=strstr(ptr,_r_delimitor))!=NULL) {
					*cptr=0;
					tmp=ptr;
					_list.add(tmp);
					ptr=cptr+sizeof(_r_delimitor);

					// BEM - start (this is so ugly)
					while (*ptr != 0x20) {
						*ptr=*ptr++;
					}
					*ptr=*ptr++;
					*ptr=*ptr++;
					// BEM - end
				}
				break;
			}
			/* Not needed anymore. BEM
			else if ((hptr=strstr(_who.c_str(),_r_delimitor))!=NULL) {
				strcpy(hdrbuf,_who.c_str())
				hptr=strstr(hdrbuf,_r_delimitor);
				*hptr=0;
				_who=hdrbuf;
			}
			*/
			// Fixes userlist garbage. BEM
			else if ((hptr=strstr(_who.c_str(),_r_delimitor3))!=NULL) {
				char whobuf[_who.length()+1];

				strcpy(whobuf,_who.c_str());
				hptr=strstr(whobuf,_r_delimitor3);
				*hptr=0;
				_who=whobuf;
			}
			// Fixes userlist garbage. BEM
			else if ((hptr=strstr(_who.c_str(),_r_delimitor4))!=NULL) {
				char whobuf[_who.length()+1];

				strcpy(whobuf,_who.c_str());
				hptr=strstr(whobuf,_r_delimitor4);
				*hptr=0;
				_who=whobuf;
			}
			else {
			}
			_mtype|=YH_MSG_ROOM_ENTER_F;	
			break;
		case pkt_room_leave:
			parse(pkt.getPayload(),pkt.getLen(),&_room,1,&_who,2);
			_mtype|=YH_MSG_ROOM_LEAVE_F;	
			break;
		case pkt_invite:
			parse(pkt.getPayload(),pkt.getLen(),&_msg,1,&_room,2,&_desc,3);
			_mtype|=YH_MSG_INVITE_F;
			if (_pflag)
				_mtype^=YH_MSG_ERR_F;	
			break;
		case pkt_user_status: {
			string
				tmp;
			parse(pkt.getPayload(),pkt.getLen(),&_who,1,&_room,2,&tmp,3,
					&_msg,4);
			if ((tmp[0]==YH_USTAT_MISC) && (tmp[1]==YH_USTAT_CUST)) {
				string
					hldr;
				hldr="(";
				hldr+=_msg;
				hldr+=")";	
				_msg="is away";
				_desc=hldr;
				_mtype|=YH_MSG_USTAT_MAWAY_F;
			}
			else if ((tmp[0]==YH_USTAT_MISC) && (tmp[1]==YH_USTAT_AUTO)) {
				_msg="is away";
				_desc="(Auto-Away)";
				_mtype|=YH_MSG_USTAT_AAWAY_F;
			}
			else {
				_mtype|=YH_MSG_USTAT_MAWAY_F;
				_msg="is away";
				switch (tmp[0]) {
					case YH_USTAT_BACK:
						_mtype^=YH_MSG_USTAT_MAWAY_F;
						_mtype|=YH_MSG_USTAT_BACK_F;
						_msg="is back"; break;
					case YH_USTAT_BRB:
						_desc="(Be Right Back)"; break;
					case YH_USTAT_BSY:
						_desc="(Busy)"; break;
					case YH_USTAT_NHOM:
						_desc="Not At Home)"; break;
					case YH_USTAT_NDSK:
						_desc="(Not At My Desk)"; break;
					case YH_USTAT_NOFC:
						_desc="(Not In The Office)"; break;
					case YH_USTAT_FON:
						_desc="(On The Phone)"; break;
					case YH_USTAT_VACA:
						_desc="(On Vacation)"; break;
					case YH_USTAT_LNCH:
						_desc="(Out To Lunch)"; break;
					case YH_USTAT_SOUT:
						_desc="(Stepped Out)"; break;
				}
			}
			break;
		}
		case pkt_thought:
			parse(pkt.getPayload(),pkt.getLen(),&_room,1,&_who,2,&_msg,3);
			strip(_msg);
			_mtype|=YH_MSG_THOUGHT_F;
			break;
		case pkt_emotion:
			parse(pkt.getPayload(),pkt.getLen(),&_room,1,&_who,2,&_msg,3);
			strip(_msg);
			_mtype|=YH_MSG_EMOTION_F;
			break;
		case pkt_speech:
			parse(pkt.getPayload(),pkt.getLen(),&_room,1,&_who,2,&_msg,3);
			strip(_msg);
			_mtype|=YH_MSG_SPEECH_F;
			break;
		case pkt_pm:
			parse(pkt.getPayload(),pkt.getLen(),&_who,1,&_msg,3);
			strip(_msg);
			_mtype|=YH_MSG_PM_F;
			break;
		case pkt_buddylist: {
			int
				rv,
				status;
			rv=parse(pkt.getPayload(),pkt.getLen(),&_msg,1,&_who,2);
			if (rv == 0)
				break;
			status=atoi(_msg.c_str());
			if (status==3) {
				_mtype|=YH_MSG_BUDDYLIST_F;
				if (rv==1)
					break;
				rv=2;
				while (parse(pkt.getPayload(),pkt.getLen(),&_who,rv) == 1){
					_list.add(_who);
					rv+=2;
				}
			}
			else if (status==1) {
				_mtype|=YH_MSG_BSTAT_ONLINE_F;
			}
			else if (status==0) {
				_mtype|=YH_MSG_BSTAT_OFFLINE_F;
			}
			break;
		}
		case pkt_dice:
			parse(pkt.getPayload(),pkt.getLen(),&_who,1,&_msg,2,&_room,3,&_desc,4);
			strip(_msg);
			_mtype|=YH_MSG_DICE_F;
			break;
		case pkt_buddy_status: {
			parse(pkt.getPayload(),pkt.getLen(),&_msg,1);
			strip(_msg);
			char buf[_msg.length()+1];
			char *ptr;
			strcpy(buf,_msg.c_str());
			if (strstr(buf,"away.")!=NULL) {
				_mtype|=YH_MSG_BSTAT_AWAY_F;
			}
			else if (strstr(buf,"back.")!=NULL) {
				_mtype|=YH_MSG_BSTAT_BACK_F;
			}
			else {
				string tmp;
				for (int i=1;parse(pkt.getPayload(),pkt.getLen(),&tmp,i);i++)
					_list.add(tmp);
			}
			if ((ptr=strchr(buf,' '))!=NULL)
				*ptr=0;
			_who=buf;
			break;
		}
		case pkt_ad:
			parse(pkt.getPayload(),pkt.getLen(),&_msg,1);
			strip(_msg);
			_mtype|=YH_MSG_ADVERTISE_F;
			break;	
		default:
			string buf;
			for (int i=1;parse(pkt.getPayload(),pkt.getLen(),&buf,i);i++)
				_list.add(buf);
	}
	if (user==_who) {
		_mtype|=YH_MSG_SELF_F;
	}
	return 0;
}

int YahooMessage::parse(const char *payload, int len, string *stor_a, 
			int seg_a, string *stor_b, int seg_b, string *stor_c, int seg_c, 
			string *stor_d, int seg_d) {
	int
		curseg=1,
		lastseg=0,
		matched=0;
	char
		*ptr,
		*bptr,
		buf[len+1];

	memcpy(buf,payload,len);
	buf[len]=0;
	bptr=buf;
	lastseg=seg_a;
	if (seg_b > lastseg)
		lastseg=seg_b;
	if (seg_c > lastseg)
		lastseg=seg_c;
	if (seg_d > lastseg)
		lastseg=seg_d;
	while (curseg <= lastseg) {
		if ((ptr=strchr(bptr,YH_IDELIMITERa))==NULL) {
			matched++;
			if (curseg==seg_a)
				(*stor_a)=bptr;
			else if (curseg==seg_b)
				(*stor_b)=bptr;
			else if (curseg==seg_c)
				(*stor_c)=bptr;
			else if (curseg==seg_d)
				(*stor_d)=bptr;
			else
				matched--;
			break;
		}
		if ((unsigned char)*(ptr+1)!=YH_IDELIMITERb) {
			bptr=ptr+1;
			continue;
		}
		*ptr=0;
		if (curseg==seg_a) {
			(*stor_a)=bptr;
			matched++;
		}
		else if (curseg==seg_b) {
			(*stor_b)=bptr;
			matched++;
		}
		else if (curseg==seg_c) {
			(*stor_c)=bptr;
			matched++;
		}
		else if (curseg==seg_d) {
			(*stor_d)=bptr;
			matched++;
		}
		curseg++;
		bptr=ptr+2;
	}
	return matched;
}

void YahooMessage::strip(string &txt) {
	unsigned int
		b=0,i=0;
	int
		strt,end;
	char
		buf[txt.length()+1];
	StrUtil
		sutil;

	while (i < txt.length()) {
		if (txt[i]!=0x1b)
			buf[b++]=txt[i++];
		else
			while(txt[i++]!=0x6d && i < txt.length());		
	}
	buf[b]=0;
	txt=buf;
	i=0;
	while (*_wordlist[i]) {
		while (sutil.findTagBounds(txt.c_str(),_wordlist[i],strt,end)==0) {
			txt.erase(strt,end-strt);
		}
		i++;
	}
}
