#ifndef __YAHOOPACKET_H__
#define __YAHOOPACKET_H__

#include <string>
#include <netinet/in.h>

#define YH_IDENT		"YCHT"
#define YH_IDELIMITERa		0x000000C0
#define YH_IDELIMITERb		0x00000080
#define	YH_ODELIMITER		0x00000001
#define YH_VER			0x00000100
#define YH_DFLAG		0x00000000

enum yhPacketType {
	pkt_undefined =		0x00000000,
	pkt_login =		0x00000001,
	pkt_logout =		0x00000002,
	pkt_room_enter =	0x00000011,
	pkt_room_leave =	0x00000012,
	pkt_invite =		0x00000017,
	pkt_user_status =	0x00000021,
	pkt_goto_user =		0x00000025,
	pkt_speech =		0x00000041,
	pkt_thought =		0x00000042,
	pkt_emotion =		0x00000043,
	pkt_ad =		0x00000044,
	pkt_pm =		0x00000045,
	pkt_buddy_status =	0x00000046,
	pkt_dice =		0x00000047,
	pkt_ping =		0x00000062,
	pkt_buddylist =		0x00000068,
	pkt_mail =		0x00000069,
	pkt_message =		0x00000070,
	pkt_misc_cmd =		0x00000071,
	pkt_grafiti =		0x00000080
};

using namespace std;

class YahooPacket {
	public:
		YahooPacket(yhPacketType ptype=pkt_undefined, 
				unsigned int flag=YH_DFLAG, const string ident=YH_IDENT,
				unsigned int ver=YH_VER);
		~YahooPacket();
		int setType(yhPacketType ptype, unsigned int flag=YH_DFLAG);
		int setPayload(const char *buf1, int len1, const char *buf2=NULL,
				int len2=0, const char *buf3=NULL, int len3=0);
		int setInbound(const char *hdr);
		string getIdent() const;
		unsigned int getVer() const;
		yhPacketType getType() const;
		unsigned int getFlag() const;
		unsigned int getLen() const;
		const char *getHeader() const { return (_pkt); };
		const char *getPayload() const { return (_pkt+_headerSize); };
		const char *serialize() const { return (_pkt); };	
		int size() const { return (getLen() + _headerSize); };
		int headerSize() const { return (_headerSize); };
	private:
		char *_pkt;
		static int _headerSize;
	
		int memsetIdent(const string ident);
		int memsetVer(unsigned int ver);
		int memsetType(unsigned int type);
		int memsetFlag(unsigned int flag);
		int memsetLen(unsigned int len);
		YahooPacket(const YahooPacket &);
		YahooPacket &operator=(const YahooPacket &);
};

#define YH_USTAT_MISC		0x00000031
#define YH_USTAT_CUST		0x00000030
#define YH_USTAT_AUTO		0x00000031
#define YH_USTAT_INVIS		0x00000032 

#define YH_USTAT_BACK		0x00000030
#define YH_USTAT_BRB		0x00000031
#define YH_USTAT_BSY		0x00000032
#define YH_USTAT_NHOM		0x00000033
#define YH_USTAT_NDSK		0x00000034
#define YH_USTAT_NOFC		0x00000035
#define YH_USTAT_FON		0x00000036
#define YH_USTAT_VACA		0x00000037
#define YH_USTAT_LNCH		0x00000038
#define YH_USTAT_SOUT		0x00000039

#endif // __YAHOOPACKET_H__
