#include "YahooProto.H"
#include "YahooPacket.H"
#include "YahooMessage.H"
#include "Logger.H"
#include <unistd.h>
#include "StrUtil.H"

extern Logger logger;

const unsigned long YahooProto::_max_sequential_read_fail(72000);

YahooProto::YahooProto(const string chatserv, int chatport, 
		const string cookieserv, int cookieport, int maxAttempts):
		_conx(NULL),
		_maxAttempts(maxAttempts),
		_secs_til_idle(1500),
		_autoaway(false),
		_thresh(0) {
	setChatServer(chatserv,chatport);
	setCookieServer(cookieserv,cookieport);
}

YahooProto::~YahooProto() {
	if (_conx)
		signOff();
}

int YahooProto::setChatServer(const string chatserv, int chatport) {
	_chatserv=chatserv;
	_chatport=chatport;
	return 0;
}

int YahooProto::setCookieServer(const string cookieserv, int cookieport) {
	_cookieserv=cookieserv;
	_cookieport=cookieport;
	return 0;
}

int YahooProto::setFriendServer(const string friendserv,int friendport) {
	_friendserv=friendserv;
	_friendport=friendport;
	return 0;
}

int YahooProto::setUserPw(const string user, const string passwd) {
	_username=user;
	_passwd=passwd;
	return 0;
}

int YahooProto::setDefaultRoom(const string room) {
	_room=room;
	return 0;
}

int YahooProto::joinRoom(const string room) {
	YahooPacket
		pkt(pkt_room_enter);
	string
		theroom;

	if (!_conx)
		return -1;

	if (room.length()==0) {
		if (_room.length()==0) {
			return -1;
		}
	}
	else {
		_room=room;
	}
	pkt.setPayload(_room.c_str(),_room.length());
	_conx->write(pkt.serialize(),pkt.size());
	return 0;
}	

int YahooProto::setUserMiscStatus(unsigned int type, const string message) {
	YahooPacket
		pkt(pkt_user_status); //BEM
	string
		buf;

	if (!_conx)
		return -1;

	//BEM buf="away away";
	buf=_room; //BEM
	buf+=YH_IDELIMITERa;
	buf+=YH_IDELIMITERb;
	buf+=YH_USTAT_MISC;
	buf+=type;
	buf+=YH_IDELIMITERa;
	buf+=YH_IDELIMITERb;
	buf+=message;
	pkt.setPayload(buf.c_str(),buf.length());
	_conx->write(pkt.serialize(),pkt.size());
	_awaymsg="is away ";
	switch (type) {
		case YH_USTAT_CUST:
			_awaymsg+='(';
			_awaymsg+=message;
			_awaymsg+=')';
			_autoaway=false;
			sendAwayMsg(_awaymsg); //BEM
			break;
		case YH_USTAT_AUTO:
			_awaymsg+="(Auto-Away)";
			sendAwayMsg(_awaymsg); //BEM
			_autoaway=true;
			break;
	}
	return 0;
}

int YahooProto::setUserStatus(unsigned int type) {
	YahooPacket
		pkt(pkt_user_status); //BEM
	string
		buf;

	if (!_conx)
		return -1;

	//BEM buf="away away";
	buf=_room; //BEM
	buf+=YH_IDELIMITERa;
	buf+=YH_IDELIMITERb;
	buf+=type;
	buf+=YH_IDELIMITERa;
	buf+=YH_IDELIMITERb;
	pkt.setPayload(buf.c_str(),buf.length());
	_conx->write(pkt.serialize(),pkt.size());
	_autoaway=false;
	_awaymsg="is away ";
	switch (type) {
		case YH_USTAT_BACK:
			_awaymsg="";
			time(&_lastpost);
			sendAwayMsg("is back"); //BEM
			break;
		case YH_USTAT_BRB:
			_awaymsg+="(Be Right Back)";
			sendAwayMsg(_awaymsg); //BEM
			break;
		case YH_USTAT_BSY:
			_awaymsg+="(Busy)";
			sendAwayMsg(_awaymsg); //BEM
			break;
		case YH_USTAT_NHOM:
			_awaymsg+="(Not At Home)";
			sendAwayMsg(_awaymsg); //BEM
			break;
		case YH_USTAT_NDSK:
			_awaymsg+="(Not At My Desk)";
			sendAwayMsg(_awaymsg); //BEM
			break;
		case YH_USTAT_NOFC:
			_awaymsg+="(Not In The Office)";
			sendAwayMsg(_awaymsg); //BEM
			break;
		case YH_USTAT_FON:
			_awaymsg+="(On The Phone)";
			sendAwayMsg(_awaymsg); //BEM
			break;
		case YH_USTAT_VACA:
			_awaymsg+="(On Vacation)";
			sendAwayMsg(_awaymsg); //BEM
			break;
		case YH_USTAT_LNCH:
			_awaymsg+="(Out To Lunch)";
			sendAwayMsg(_awaymsg); //BEM
			break;
		case YH_USTAT_SOUT:
			_awaymsg+="(Stepped Out)";
			sendAwayMsg(_awaymsg); //BEM
			break;
	}
	_autoaway=false;
	return 0;
}

int YahooProto::gotoUser(const string who) {
	YahooPacket
		pkt(pkt_goto_user);

	if (!_conx)
		return -1;

	pkt.setPayload(who.c_str(),who.length());
	_conx->write(pkt.serialize(),pkt.size());
	time(&_lastpost);
	return 0;
}	

int YahooProto::sendMessage(const string message) {
	YahooPacket
		pkt(pkt_speech);
	string
		buf;
	
	if (!_conx)
		return -1;

	if (_awaymsg.length()) //BEM
		setUserStatus(); //BEM

	buf=_room;
	buf+=YH_ODELIMITER;
	buf+=message;
	pkt.setPayload(buf.c_str(),buf.length());
	_conx->write(pkt.serialize(),pkt.size());
	time(&_lastpost);
	return 0;
}

int YahooProto::sendPrivateMessage(const string who, const string message,
		bool autoreply, const string from) {
	YahooPacket
		pkt(pkt_pm);
	string
		buf;

	if (!_conx)
		return -1;

	if (from.length()) {
		buf=from;
	}
	else {
		buf=_username;
	}
	buf+=YH_ODELIMITER;
	buf+=who;
	buf+=YH_ODELIMITER;
	buf+=message;
	pkt.setPayload(buf.c_str(),buf.length());
	_conx->write(pkt.serialize(),pkt.size());
	if (!autoreply) {
		time(&_lastpost);
	}
	return 0;
}

int YahooProto::sendEmote(const string message) {
	YahooPacket
		pkt(pkt_emotion);
	string
		buf;
	
	if (!_conx)
		return -1;

	if (_awaymsg.length()) //BEM
		setUserStatus(); //BEM

	buf=_room;
	buf+=YH_ODELIMITER;
	buf+=message;
	pkt.setPayload(buf.c_str(),buf.length());
	_conx->write(pkt.serialize(),pkt.size());
	time(&_lastpost);
	return 0;
}

int YahooProto::sendAwayMsg(const string message) { //BEM
	YahooPacket
		pkt(pkt_emotion);
	string
		buf;
	
	if (!_conx)
		return -1;

	buf=_room;
	buf+=YH_ODELIMITER;
	buf+=message;
	pkt.setPayload(buf.c_str(),buf.length());
	_conx->write(pkt.serialize(),pkt.size());
	sleep(1);
	return 0;
}

int YahooProto::sendThought(const string message) {
	YahooPacket
		pkt(pkt_emotion);
	string
		buf;
	
	if (!_conx)
		return -1;

	if (_awaymsg.length()) //BEM
		setUserStatus(); //BEM

	buf=_room;
	buf+=YH_ODELIMITER;
	buf+=". o O ( ";
	buf+=message;
	buf+=" )";
	pkt.setPayload(buf.c_str(),buf.length());
	_conx->write(pkt.serialize(),pkt.size());
	time(&_lastpost);
	return 0;
}

int YahooProto::rollDice(const string message) {
	YahooPacket
		pkt(pkt_misc_cmd);
	string
		buf;
	
	if (!_conx)
		return -1;
	buf="roll ";
	buf+=message;
	pkt.setPayload(buf.c_str(),buf.length());
	_conx->write(pkt.serialize(),pkt.size());
	time(&_lastpost);
	return 0;
}

int YahooProto::sendInvite(const string who) {
	YahooPacket
		pkt(pkt_misc_cmd);
	string
		buf;
	
	if (!_conx)
		return -1;
	buf="invite ";
	buf+=who;
	pkt.setPayload(buf.c_str(),buf.length());
	_conx->write(pkt.serialize(),pkt.size());
	time(&_lastpost);
	return 0;
}

int YahooProto::ignoreUser(const string message) {
	YahooPacket
		pkt(pkt_misc_cmd);
	string
		buf;
	
	if (!_conx)
		return -1;
	buf="ignore ";
	buf+=message;
	buf+=YH_ODELIMITER;
	pkt.setPayload(buf.c_str(),buf.length());
	_conx->write(pkt.serialize(),pkt.size());
	return 0;
}

bool YahooProto::getSpeech(YahooMessage &msg) {
	if (_spchQ.empty())
		return false;
	msg=_spchQ.front();
	_spchQ.pop();
	return true;
}

bool YahooProto::getEvent(YahooMessage &msg) {
	if (_evtQ.empty())
		return false;
	msg=_evtQ.front();
	_evtQ.pop();
	return true;
}

int YahooProto::signOn() {
	YahooMessage
		blmsg,
		msg;
	YahooPacket
		dpkt;
	int
		attempt=0,
		rval;
	bool
		signedOn=false,
		gotBL=false,
		gotLogin=false;
	string
		tmp,
		buf;

	if ((rval=fetchCookie())) {
		logger.log("YahooProto::signOn: fetchCookie failed: %d",rval);
		return -1;
	}
	if ((rval=openConnection())) {
		logger.log("YahooProto::signOn: openConnection failed: %d",rval);
		return -2;
	}
	while ((!signedOn) && (attempt<_maxAttempts)) {
		if (attempt++) {
			sleep(1);
		}
		if (sendLogin()) {
			logger.log("YahooProto::signOn: error writing to socket during login");
			continue;
		}
		while ((!signedOn) && (readPacket(dpkt)==0)) {
			if (!getEvent(msg)) {
				logger.log("YahooProto::signOn: didnt get an event packet, "
						"retrying");
				printPacket(dpkt);
				break;;
			}
 			if (msg.isBuddyList()) {
				logger.log("YahooProto::signOn: got the buddylist");
				gotBL=true;
				blmsg=msg;
				if (gotLogin) 
					signedOn=true;
				continue;
			}
 			if (msg.isLogin()) {
				// bad words go here
				// do something with it here
				logger.log("YahooProto::signOn: got the login");
				gotLogin=true;
				if (gotBL) 
					signedOn=true;
				continue;
			}
			logger.log("YahooProto::signOn: weird event from flawhoo");
			printPacket(dpkt);
			break;
		}
	}
	if (!signedOn) {
		logger.log("YahooProto::signOn: all %d attempts to signon have failed",
				_maxAttempts);
		return -1;
	}
	attempt=0;
	if (_room.length()) { // done this way in case this ever becomes msg clnt
		while (attempt<_maxAttempts) {
			if (attempt++) {
				sleep(1);
			}
			joinRoom(_room);
			readPacket(dpkt);
			if (!getEvent(msg) || !msg.isRoomList()) {
				logger.log("YahooProto::signOn: didnt get a joinroom packet, "
						"retrying");
				printPacket(dpkt);
				continue;
			}
			putEvent(msg);
			break;
		}
		if (attempt==_maxAttempts) {
			logger.log("YahooProto::signOn: all %d attempts to join the room "
					"have failed", _maxAttempts);
			return -2;
		}
	}
	putEvent(blmsg);
#ifdef MULTITHREAD
	pthread_create(&_listenerThr,NULL,(void *(*)(void *))packetListener, this);
#endif
	return 0;
}

int YahooProto::signOnWithMessages() {
	int 
		rval;

	if ((rval=openConnection())) {
		logger.log("YahooProto::signOn: openConnection failed: %d",rval);
		return -1;
	}
	if ((rval=sendLogin())) {
		logger.log("YahooProto::signOn: sendLogin failed: %d:",rval);
		return -2;
	}
#ifdef MULTITHREAD
	pthread_create(&_listenerThr,NULL,(void *(*)(void *))packetListener, this);
#endif
	return 0;
}

int YahooProto::signOff() {
	if (!_conx)
		return -1;
	sendLogout();
#ifdef MULTITHREAD
	pthread_cancel(_listenerThr);
#endif
	closeConnection();
	return 0;
}

int YahooProto::sendLogin() {
	YahooPacket
		pkt(pkt_login);
	int
		rv;

	if (!_conx) {
		return -999;
	}
	pkt.setPayload(_username.c_str(),_username.length(),_cookie.c_str(),
		_cookie.length());
	if ((rv=_conx->write(pkt.serialize(),pkt.size())) != pkt.size()) {
		if (rv == 0)
			rv=-998;
		return rv;
	}
	_lastping=time(&_lastpost);
	return 0;
}

int YahooProto::sendLogout() {
	YahooPacket
		pkt(pkt_logout);
	
	if (!_conx)
		return -1;
	pkt.setPayload(_username.c_str(),_username.length());
	_conx->write(pkt.serialize(),pkt.size());
	return 0;
}

int YahooProto::fetchCookie() {
	Socket
		sock(_cookieserv, _cookieport);
	int 
		i=0;
	char
		*ptr,
		*tptr,
		*ptr2,
		*tptr2,
		loginString[256],
		returnData[4096];
	string
		username(_username),
		passwd(_passwd);
	StrUtil
		sutl;	

	sutl.straightReplace(" ","%20",username, true);
	sutl.straightReplace(" ","%20",passwd, true);
	sprintf(loginString, "GET /config/login?login=%s&passwd=%s&.chkP=Y "
			"HTTP/1.0\r\nAccept: */*\r\n\r\n\r\n",
			username.c_str(), passwd.c_str());

	if (sock.open()) {
		return -1;
	}
	if (sock.write(loginString,strlen(loginString))!=(int)strlen(loginString)) {
		return -2;
	}
	if ((i=sock.read(returnData,sizeof(returnData),90)) <= 0) {
		logger.log("YahooProto::fetchCookie: read returned %d",i);
		return -3;
	}

	if ((ptr=strstr(returnData,"Set-Cookie: Y"))==NULL) {
		logger.log("YahooProto::fetchCookie: no cookie in da cookie!");
		return -4;
	}
	if ((ptr2=strstr(returnData,"Set-Cookie: T"))==NULL) {
		logger.log("YahooProto::fetchCookie: no cookie in da cookie!");
		return -4;
	}

	if ((tptr=strchr(ptr,';'))==NULL) {
		logger.log("YahooProto::fetchCookie: no viable end to the cookie");
		return -5;
	}
	*tptr=0;	
	if ((tptr2=strchr(ptr2,';'))==NULL) {
		logger.log("YahooProto::fetchCookie: no viable end to the cookie");
		return -5;
	}
	*tptr2=0;	

	while(((*ptr)!='Y') && (ptr<tptr))
		ptr++;

	while(((*ptr2)!='T') && (ptr2<tptr2))
		ptr2++;

	if ((*ptr)!='Y') {
		logger.log("YahooProto::fetchCookie: no viable begin to the cookie");
		return -6;
	}
	if ((*ptr2)!='T') {
		logger.log("YahooProto::fetchCookie: no viable begin to the cookie");
		return -6;
	}

	_cookie=ptr;
	_cookie+="; ";
	_cookie+=ptr2;
	return 0;
}

int YahooProto::ping() {
	YahooPacket
		pkt(pkt_ping);

	if (!_conx)
		return -1;
	_conx->write(pkt.serialize(),pkt.size());
	return 0;
}

int YahooProto::readPacket(YahooPacket &pkt,int waitsecs,time_t timestamp) {
	int
		rcvd;
	YahooMessage
		*msg=NULL;
	char
		hdr[pkt.headerSize()],
		*ptr,
		*payload;

	if (!_conx)
		return -1;
	if ((rcvd=_conx->read(hdr,sizeof(hdr),waitsecs))!=(signed)sizeof(hdr)) {
		if (rcvd < 0)
			return -2;
		ptr=hdr+rcvd;
		rcvd=_conx->read(hdr+rcvd,(sizeof(hdr)-rcvd),-1);
	}
	pkt.setInbound(hdr);
	if (pkt.getLen()) {
		if ((payload=new char[pkt.getLen()+1])==NULL) {
        	logger.log("YahooProto::readPacket: error allocating buffer");
        	return -3;
    	}
		if ((rcvd=_conx->read(payload,pkt.getLen(),-1))!=
				(signed)pkt.getLen()) {
			int newr;
			logger.log("YahooProto::readPacket: appending short packet..."
				" %d (%d) [%d]", rcvd,(signed)pkt.getLen(),(signed)pkt.getLen()-rcvd);
			if (rcvd<0)
				rcvd=0;
			while ((newr=_conx->read((payload+rcvd),(pkt.getLen()-rcvd),
					waitsecs)) !=((signed)pkt.getLen()-rcvd)) {
				logger.log("YahooProto::readPacket: short packet: %d+%d=%d != %d",rcvd,newr,rcvd+newr,pkt.getLen());
			}
		}
		pkt.setPayload(payload,pkt.getLen());
		delete[](payload);
	}
	msg=new YahooMessage(pkt,_username,timestamp);
	if (msg->isAnySpeech()) {
		putSpeech(*msg);
	}
	else {
		putEvent(*msg);
	}
	delete(msg);
	return 0;
}

int YahooProto::packetReader() {
	time_t
		curtime;
	YahooPacket
		pkt;

	time(&curtime);
	if (curtime > (_lastping + 600)) {
       		ping();
		_lastping=curtime;
	}
	if ((curtime > (_lastpost + _secs_til_idle))) {
		if ((!_autoaway) && (!_awaymsg.length())) {
			setUserMiscStatus(YH_USTAT_AUTO);
		}
	}
	else if (_autoaway) {
		setUserStatus();
		_autoaway=false;
	}
	if (readPacket(pkt,0,curtime)) {
		if (_thresh++ > _max_sequential_read_fail) {
			YahooMessage
				msg;
			msg.hasLostConnection();
			putEvent(msg);
			return -1;		
		}
		else {
			return 1;
		}
	}
	else {
		_thresh=0;
		if (_awaymsg.length()) {
			YahooMessage msg(pkt,_username);
			if ((!msg.isSelf()) && (msg.isPM()))
				sendPrivateMessage(msg.who(),_awaymsg,true);
		}
		if (_follow.length()) {
			YahooMessage msg(pkt,_username);
			if ((msg.isRoomLeave()) &&
				(!strcasecmp(_follow.c_str(),msg.who().c_str()))) {
				gotoUser(msg.who());
			}
		}
	}
	return 0;
}

void *YahooProto::packetListener(YahooProto *this_) {
    while (1) {
		if (this_->packetReader()==1)
			usleep(50000);
    }
}

int YahooProto::openConnection() {
	if (_conx)
		closeConnection();
	_conx=new Socket(_chatserv, _chatport);	
	_conx->setNonBlock();
	if (_conx->open()) {
		return -2;
	}
	return 0;
}

int YahooProto::closeConnection() {
	if (!_conx)
		return -1;
	_conx->close();
	delete(_conx);
	_conx=NULL;
	return 0;
}

void YahooProto::printPacket(const YahooPacket &pkt) {
	logger.log("YahooProto::printPacket: %s %X %X %X %X",pkt.getIdent().c_str(),
			pkt.getVer(), pkt.getType(),pkt.getFlag(),pkt.getLen());
}
