#ifndef __YAHOOPROTO_H__
#define __YAHOOPROTO_H__

#include "Socket.H"
#include "YahooMessage.H"
#include <pthread.h>
#include <time.h>
#include <string>
#include <queue>

class YahooProto {
	public:
		YahooProto(const string chatserv="cs6.chat.sc5.yahoo.com",                      //known server to be up 99% of the time - CIA 19-Oct.-2003 14:15
				int chatport=8002, const string cookieserv="login.yahoo.com",   
				int cookieport=80, int maxAttempts=2);
		~YahooProto();
		int setChatServer(const string chatserv, int chatport);
		int setCookieServer(const string cookieserv,int cookieport);
		int setFriendServer(const string friendserv,int friendport);
		int setUserPw(const string user, const string passwd);
		int setDefaultRoom(const string room);
		int joinRoom(const string room="");
		int setUserMiscStatus(unsigned int type, const string message="");
		int setUserStatus(unsigned int type=YH_USTAT_BACK);
		void setFollow(const string who="") { _follow=who; }
		string getFollow() { return _follow; }
		int gotoUser(const string who);
		int sendMessage(const string message);
		int sendPrivateMessage(const string who, const string message, 
				bool autoreply=false, const string from="");
		int sendEmote(const string message);
		int sendAwayMsg(const string message); //BEM
		int sendThought(const string message);
		int rollDice(const string message);
		int sendInvite(const string who);
		int ignoreUser(const string who);
		int packetReader();
		bool getSpeech(YahooMessage &msg);
		bool getEvent(YahooMessage &msg);
		int signOn();
		// use signOn or the  following funcs
		int fetchCookie();
		int signOnWithMessages();
		int signOff();
		int putEvent(const YahooMessage &msg) { _evtQ.push(msg); return 0; }
	protected:
		int sendLogin();
		int sendLogout();
		int ping();
		int putSpeech(const YahooMessage &msg) { _spchQ.push(msg); return 0; }
		int readPacket(int waitsecs=-1, time_t timestamp=0) {
			YahooPacket pkt; return (readPacket(pkt,waitsecs,timestamp)); }
		int readPacket(YahooPacket &pkt,int waitsecs=-1, time_t timestamp=0);
		static void *packetListener(YahooProto *this_);
		int openConnection();
		int closeConnection();
	private:
		static const unsigned long _max_sequential_read_fail;
		Socket *_conx;
		pthread_t _listenerThr;
		time_t _lastpost;
		time_t _lastping;
		string _chatserv;
		int _chatport;
		string _cookieserv;
		int _cookieport;
		string _friendserv;
		int _friendport;
		int _maxAttempts;
		int _secs_til_idle;
		string _cookie;
		string _username;
		string _passwd;
		string _room;
		string _awaymsg;
		string _follow;
		bool _autoaway;
		queue<YahooMessage> _spchQ;
		queue<YahooMessage> _evtQ;
		unsigned int _thresh;

		YahooProto(const YahooProto &);	
		YahooProto &operator=(const YahooProto &);	
		void printPacket(const YahooPacket &pkt);
};

#endif // __YAHOOPROTO_H__
